@@ -49,9 +49,9 @@ static const union av_perm_to_string_data {
 	}
 };
 static const struct av_perm_to_string {
-	u16 tclass;
-	u16 nameidx;
-	u32 value;
+	uint16_t tclass;
+	uint16_t nameidx;
+	uint32_t value;
 } av_perm_to_string[] = {
 #define S_(c, v, s) { c, offsetof(union av_perm_to_string_data, L1(__LINE__)), v },
 #include "av_perm_to_string.h"
@@ -77,7 +77,7 @@ static const union class_to_string_data {
 #undef  S_
 	}
 };
-static const u16 class_to_string[] = {
+static const uint16_t class_to_string[] = {
 #define S_(s) offsetof(union class_to_string_data, L1(__LINE__)),
 #include "class_to_string.h"
 #undef  S_
@@ -112,7 +112,7 @@ static const union common_perm_to_string {
 	struct {
 #define TB_(s) struct {
 #define TE_(s) } s##_part;
-#define S_(s) u16 L1(__LINE__)
+#define S_(s) uint16_t L1(__LINE__)
 #define L1(l) L2(l)
 #define L2(l) field_##l;
 #include "common_perm_to_string.h"
@@ -122,7 +122,7 @@ static const union common_perm_to_string {
 #undef L1
 #undef L2
 	};
-	u16 data[0];
+	uint16_t data[0];
 } common_perm_to_string = {
 	{
 #define TB_(s) {
@@ -140,11 +140,11 @@ static const union common_perm_to_string {
 };
 
 static const struct av_inherit {
-	u16 tclass;
-	u16 common_pts_idx;
-	u32 common_base;
+	uint16_t tclass;
+	uint16_t common_pts_idx;
+	uint32_t common_base;
 } av_inherit[] = {
-#define S_(c, i, b) { c, offsetof(union common_perm_to_string, common_##i##_perm_to_string_part)/sizeof(u16), b },
+#define S_(c, i, b) { c, offsetof(union common_perm_to_string, common_##i##_perm_to_string_part)/sizeof(uint16_t), b },
 #include "av_inherit.h"
 #undef S_
 };
@@ -168,17 +168,17 @@ struct avc_node {
 
 struct avc_cache {
 	struct avc_node *slots[AVC_CACHE_SLOTS];
-	u_int32_t lru_hint;	/* LRU hint for reclaim scan */
-	u_int32_t active_nodes;
-	u_int32_t latest_notif;	/* latest revocation notification */
+	uint32_t lru_hint;	/* LRU hint for reclaim scan */
+	uint32_t active_nodes;
+	uint32_t latest_notif;	/* latest revocation notification */
 };
 
 struct avc_callback_node {
-	int (*callback) (u_int32_t event, security_id_t ssid,
+	int (*callback) (uint32_t event, security_id_t ssid,
 			 security_id_t tsid,
 			 security_class_t tclass, access_vector_t perms,
 			 access_vector_t * out_retained);
-	u_int32_t events;
+	uint32_t events;
 	security_id_t ssid;
 	security_id_t tsid;
 	security_class_t tclass;
@@ -272,12 +272,29 @@ int sidget(security_id_t sid)
 int sidput(security_id_t sid)
 {
 	int rc;
+	if (!sid)
+	    return 0;
 	avc_get_lock(avc_lock);
 	rc = sid_dec_refcnt(sid);
 	avc_release_lock(avc_lock);
 	return rc;
 }
 
+int avc_get_initial_sid(const char * name, security_id_t * sid)
+{
+	int rc;
+	security_context_t con;
+
+	rc = security_get_initial_context_raw(name, &con);
+	if (rc < 0)
+		return rc;
+	rc = avc_context_to_sid_raw(con, sid);
+
+	freecon(con);
+
+	return rc;
+}
+
 int avc_init(const char *prefix,
 	     const struct avc_memory_callback *mem_cb,
 	     const struct avc_log_callback *log_cb,
@@ -402,7 +419,7 @@ static inline struct avc_node *avc_reclaim_node(void)
 {
 	struct avc_node *prev, *cur;
 	int try;
-	u_int32_t hvalue;
+	uint32_t hvalue;
 
 	hvalue = avc_cache.lru_hint;
 	for (try = 0; try < 2; try++) {
@@ -795,8 +812,8 @@ static inline int check_avc_ratelimit(void)
  */
 static void avc_dump_av(security_class_t tclass, access_vector_t av)
 {
-	const u16 *common_pts_idx = 0;
-	u_int32_t common_base = 0, perm;
+	const uint16_t *common_pts_idx = 0;
+	uint32_t common_base = 0, perm;
 	unsigned int i, i2;
 
 	if (av == 0) {
@@ -982,9 +999,12 @@ int avc_has_perm_noaudit(security_id_t ssid,
 
 	denied = requested & ~(ae->avd.allowed);
 
-	if ((!requested || denied) && avc_enforcing) {
-		errno = EACCES;
-		rc = -1;
+	if (!requested || denied) {
+		if (avc_enforcing) {
+			errno = EACCES;
+			rc = -1;
+		} else
+			ae->avd.allowed |= requested;
 	}
 
       out:
@@ -1006,12 +1026,37 @@ int avc_has_perm(security_id_t ssid, security_id_t tsid,
 	return rc;
 }
 
-int avc_add_callback(int (*callback) (u_int32_t event, security_id_t ssid,
+int avc_compute_create(security_id_t ssid,  security_id_t tsid,
+		       security_class_t tclass, security_id_t *newsid)
+{
+	int rc;
+	*newsid = NULL;
+	avc_get_lock(avc_lock);
+	if (ssid->refcnt > 0 && tsid->refcnt > 0) {
+		security_context_t ctx = NULL;
+		rc = security_compute_create_raw(ssid->ctx, tsid->ctx, tclass,
+						 &ctx);
+		if (rc)
+			goto out;
+		rc = sidtab_context_to_sid(&avc_sidtab, ctx, newsid);
+		if (!rc)
+			(*newsid)->refcnt++;
+		freecon(ctx);
+	} else {
+		errno = EINVAL;	/* bad reference count */
+		rc = -1;
+	}
+out:
+	avc_release_lock(avc_lock);
+	return rc;
+}
+
+int avc_add_callback(int (*callback) (uint32_t event, security_id_t ssid,
 				      security_id_t tsid,
 				      security_class_t tclass,
 				      access_vector_t perms,
 				      access_vector_t * out_retained),
-		     u_int32_t events, security_id_t ssid,
+		     uint32_t events, security_id_t ssid,
 		     security_id_t tsid,
 		     security_class_t tclass, access_vector_t perms)
 {
@@ -1041,7 +1086,7 @@ static inline int avc_sidcmp(security_id_t x, security_id_t y)
 	return (x == y || x == SECSID_WILD || y == SECSID_WILD);
 }
 
-static inline void avc_update_node(u_int32_t event, struct avc_node *node,
+static inline void avc_update_node(uint32_t event, struct avc_node *node,
 				   access_vector_t perms)
 {
 	switch (event) {
@@ -1067,7 +1112,7 @@ static inline void avc_update_node(u_int32_t event, struct avc_node *node,
 	}
 }
 
-static int avc_update_cache(u_int32_t event, security_id_t ssid,
+static int avc_update_cache(uint32_t event, security_id_t ssid,
 			    security_id_t tsid, security_class_t tclass,
 			    access_vector_t perms)
 {
@@ -1105,10 +1150,10 @@ static int avc_update_cache(u_int32_t event, security_id_t ssid,
  * This should not be called directly; use the individual event
  * functions instead.
  */
-static int avc_control(u_int32_t event, security_id_t ssid,
+static int avc_control(uint32_t event, security_id_t ssid,
 		       security_id_t tsid, security_class_t tclass,
 		       access_vector_t perms,
-		       u_int32_t seqno, access_vector_t * out_retained)
+		       uint32_t seqno, access_vector_t * out_retained)
 {
 	struct avc_callback_node *c;
 	access_vector_t tretained = 0, cretained = 0;
@@ -1167,7 +1212,7 @@ static int avc_control(u_int32_t event, security_id_t ssid,
  */
 int avc_ss_grant(security_id_t ssid, security_id_t tsid,
 		 security_class_t tclass, access_vector_t perms,
-		 u_int32_t seqno)
+		 uint32_t seqno)
 {
 	return avc_control(AVC_CALLBACK_GRANT,
 			   ssid, tsid, tclass, perms, seqno, 0);
@@ -1188,7 +1233,7 @@ int avc_ss_grant(security_id_t ssid, security_id_t tsid,
  */
 int avc_ss_try_revoke(security_id_t ssid, security_id_t tsid,
 		      security_class_t tclass,
-		      access_vector_t perms, u_int32_t seqno,
+		      access_vector_t perms, uint32_t seqno,
 		      access_vector_t * out_retained)
 {
 	return avc_control(AVC_CALLBACK_TRY_REVOKE,
@@ -1208,7 +1253,7 @@ int avc_ss_try_revoke(security_id_t ssid, security_id_t tsid,
  */
 int avc_ss_revoke(security_id_t ssid, security_id_t tsid,
 		  security_class_t tclass, access_vector_t perms,
-		  u_int32_t seqno)
+		  uint32_t seqno)
 {
 	return avc_control(AVC_CALLBACK_REVOKE,
 			   ssid, tsid, tclass, perms, seqno, 0);
@@ -1218,7 +1263,7 @@ int avc_ss_revoke(security_id_t ssid, security_id_t tsid,
  * avc_ss_reset - Flush the cache and revalidate migrated permissions.
  * @seqno: policy sequence number
  */
-int avc_ss_reset(u_int32_t seqno)
+int avc_ss_reset(uint32_t seqno)
 {
 	int rc;
 
@@ -1243,7 +1288,7 @@ int avc_ss_reset(u_int32_t seqno)
  */
 int avc_ss_set_auditallow(security_id_t ssid, security_id_t tsid,
 			  security_class_t tclass, access_vector_t perms,
-			  u_int32_t seqno, u_int32_t enable)
+			  uint32_t seqno, uint32_t enable)
 {
 	if (enable)
 		return avc_control(AVC_CALLBACK_AUDITALLOW_ENABLE,
@@ -1264,7 +1309,7 @@ int avc_ss_set_auditallow(security_id_t ssid, security_id_t tsid,
  */
 int avc_ss_set_auditdeny(security_id_t ssid, security_id_t tsid,
 			 security_class_t tclass, access_vector_t perms,
-			 u_int32_t seqno, u_int32_t enable)
+			 uint32_t seqno, uint32_t enable)
 {
 	if (enable)
 		return avc_control(AVC_CALLBACK_AUDITDENY_ENABLE,
@@ -1303,7 +1348,7 @@ security_class_t string_to_security_class(const char *s)
 
 access_vector_t string_to_av_perm(security_class_t tclass, const char *s)
 {
-	const u16 *common_pts_idx = 0;
+	const uint16_t *common_pts_idx = 0;
 	access_vector_t perm, common_base = 0;
 	unsigned int i;
 
@@ -1338,9 +1383,108 @@ access_vector_t string_to_av_perm(security_class_t tclass, const char *s)
 	return 0;
 }
 
+const char *security_class_to_string(security_class_t tclass)
+{
+	tclass = (tclass > 0 && tclass < NCLASSES) ? tclass : 0;
+	return class_to_string_data.str + class_to_string[tclass];
+}
+
+const char *security_av_perm_to_string(security_class_t tclass,
+				       access_vector_t av)
+{
+	const uint16_t *common_pts_idx = 0;
+	access_vector_t common_base = 0;
+	unsigned int i;
+
+	if (!av)
+		return NULL;
+
+	for (i = 0; i < ARRAY_SIZE(av_inherit); i++) {
+		if (av_inherit[i].tclass == tclass) {
+			common_pts_idx =
+			    &common_perm_to_string.data[av_inherit[i].
+							common_pts_idx];
+			common_base = av_inherit[i].common_base;
+			break;
+		}
+	}
+
+	if (av < common_base) {
+		i = 0;
+		while (!(av & 1)) {
+			av >>= 1;
+			i++;
+		}
+		return common_perm_to_string_data.str + common_pts_idx[i];
+	}
+
+	for (i = 0; i < NVECTORS; i++) {
+		if (av_perm_to_string[i].tclass == tclass &&
+		    av_perm_to_string[i].value == av)
+			return av_perm_to_string_data.str
+				+ av_perm_to_string[i].nameidx;
+	}
+
+	return NULL;
+}
+
+int security_av_string(security_class_t tclass, access_vector_t av, char **res)
+{
+	unsigned int i = 0;
+	size_t len = 5;
+	access_vector_t tmp = av;
+	int rc = 0;
+	const char *str;
+	char *ptr;
+
+	/* first pass computes the required length */
+	while (tmp) {
+		if (tmp & 1) {
+			str = security_av_perm_to_string(tclass, av & (1<<i));
+			if (str)
+				len += strlen(str) + 1;
+			else {
+				rc = -1;
+				errno = EINVAL;
+				goto out;
+			}
+		}
+		tmp >>= 1;
+		i++;
+	}
+
+	*res = malloc(len);
+	if (!*res) {
+		rc = -1;
+		goto out;
+	}
+
+	/* second pass constructs the string */
+	i = 0;
+	tmp = av;
+	ptr = *res;
+
+	if (!av) {
+		sprintf(ptr, "null");
+		goto out;
+	}
+
+	ptr += sprintf(ptr, "{ ");
+	while (tmp) {
+		if (tmp & 1)
+			ptr += sprintf(ptr, "%s ", security_av_perm_to_string(
+					       tclass, av & (1<<i)));
+		tmp >>= 1;
+		i++;
+	}
+	sprintf(ptr, "}");
+out:
+	return rc;
+}
+
 void print_access_vector(security_class_t tclass, access_vector_t av)
 {
-	const u16 *common_pts_idx = 0;
+	const uint16_t *common_pts_idx = 0;
 	access_vector_t common_base = 0;
 	unsigned int i, i2, perm;
 
