@@ -11,12 +11,13 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <asm/types.h>
+#include <linux/types.h>
 #include <linux/netlink.h>
 #include "selinux_netlink.h"
 #include "avc_internal.h"
@@ -163,6 +164,12 @@ int avc_netlink_check_nb(void)
 				    ("%s:  received setenforce notice (enforcing=%d)\n",
 				     avc_prefix, msg->val);
 				avc_enforcing = msg->val;
+				if (avc_enforcing && (rc = avc_ss_reset(0)) < 0) {
+					avc_log
+					    ("%s:  cache reset returned %d (errno %d)\n",
+					     avc_prefix, rc, errno);
+					goto out;
+				}
 				break;
 			}
 
@@ -263,6 +270,12 @@ void avc_netlink_loop(void)
 				    ("%s:  received setenforce notice (enforcing=%d)\n",
 				     avc_prefix, msg->val);
 				avc_enforcing = msg->val;
+				if (avc_enforcing && (ret = avc_ss_reset(0)) < 0) {
+					avc_log
+					    ("%s:  cache reset returned %d (errno %d)\n",
+					     avc_prefix, ret, errno);
+					goto out;
+				}
 				break;
 			}
 
