@@ -9,8 +9,10 @@
 
 int fsetfilecon_raw(int fd, security_context_t context)
 {
-	return fsetxattr(fd, XATTR_NAME_SELINUX, context, strlen(context)+1, 0);
+	return fsetxattr(fd, XATTR_NAME_SELINUX, context, strlen(context) + 1,
+			 0);
 }
+
 hidden_def(setfilecon_raw)
 
 int fsetfilecon(int fd, security_context_t context)
@@ -18,13 +20,12 @@ int fsetfilecon(int fd, security_context_t context)
 	int ret;
 	security_context_t rcontext = context;
 
-	if (context_translations && trans_to_raw_context(context, &rcontext))
+	if (selinux_trans_to_raw_context(context, &rcontext))
 		return -1;
 
- 	ret = fsetfilecon_raw(fd, rcontext);
+	ret = fsetfilecon_raw(fd, rcontext);
 
-	if (context_translations)
-		freecon(rcontext);
+	freecon(rcontext);
 
 	return ret;
 }
