@@ -165,7 +165,6 @@ hidden_def(selinux_mkload_policy)
  * We only need the hardcoded definition for the initial mount 
  * required for the initial policy load.
  */
-#define SELINUXMNT "/selinux/"
 int selinux_init_load_policy(int *enforce)
 {
 	int rc = 0, orig_enforce = 0, seconfig = -2, secmdline = -1;
@@ -227,6 +226,7 @@ int selinux_init_load_policy(int *enforce)
 			 */
 			*enforce = 0;
 		}
+		fprintf(stderr, "Mount failed for selinuxfs on %s:  %s\n", SELINUXMNT, strerror(errno));
 		goto noload;
 	}
 	set_selinuxmnt(SELINUXMNT);
