@@ -19,6 +19,11 @@ int security_policyvers(void)
 	char buf[20];
 	unsigned vers = DEFAULT_POLICY_VERSION;
 
+	if (!selinux_mnt) {
+		errno = ENOENT;
+		return -1;
+	}
+
 	snprintf(path, sizeof path, "%s/policyvers", selinux_mnt);
 	fd = open(path, O_RDONLY);
 	if (fd < 0) {
@@ -28,7 +33,7 @@ int security_policyvers(void)
 			return -1;
 	}
 	memset(buf, 0, sizeof buf);
-	ret = read(fd, buf, sizeof buf-1);
+	ret = read(fd, buf, sizeof buf - 1);
 	close(fd);
 	if (ret < 0)
 		return -1;
@@ -38,5 +43,5 @@ int security_policyvers(void)
 
 	return vers;
 }
-hidden_def(security_policyvers)
 
+hidden_def(security_policyvers)
