@@ -17,7 +17,6 @@
 #define SELINUXTAG "SELINUX="
 #define SETLOCALDEFS "SETLOCALDEFS="
 #define REQUIRESEUSERS "REQUIRESEUSERS="
-#define CACHETRANSTAG "CACHETRANS="
 
 /* Indices for file paths arrays. */
 #define BINPOLICY         0
@@ -38,7 +37,8 @@
 #define NETFILTER_CONTEXTS    15
 #define FILE_CONTEXTS_HOMEDIR 16
 #define FILE_CONTEXTS_LOCAL 17
-#define NEL               18
+#define SECURETTY_TYPES   18
+#define NEL               19
 
 /* New layout is relative to SELINUXDIR/policytype. */
 static char *file_paths[NEL];
@@ -64,44 +64,16 @@ static const uint16_t file_path_suffixes_idx[NEL] = {
 #undef S_
 };
 
-/* Old layout had fixed locations. */
-#define SECURITYCONFIG "/etc/sysconfig/selinux"
-#define SECURITYDIR "/etc/security"
-static const union compat_file_path_data {
-	struct {
-#define S_(n, s) char L1(__LINE__)[sizeof(s)];
-#include "compat_file_path.h"
-#undef S_
-	};
-	char str[0];
-} compat_file_path_data = {
-	{
-#define S_(n, s) s,
-#include "compat_file_path.h"
-#undef S_
-	}
-};
-static const uint16_t compat_file_path_idx[NEL] = {
-#define S_(n, s) [n] = offsetof(union compat_file_path_data, L1(__LINE__)),
-#include "compat_file_path.h"
-#undef S_
-};
-
 #undef L1
 #undef L2
 
-static int use_compat_file_path;
-
 int selinux_getenforcemode(int *enforce)
 {
 	int ret = -1;
 	FILE *cfg = fopen(SELINUXCONFIG, "r");
-	char *buf;
-	int len = sizeof(SELINUXTAG) - 1;
-	if (!cfg) {
-		cfg = fopen(SECURITYCONFIG, "r");
-	}
 	if (cfg) {
+		char *buf;
+		int len = sizeof(SELINUXTAG) - 1;
 		buf = malloc(selinux_page_size);
 		if (!buf) {
 			fclose(cfg);
@@ -166,12 +138,6 @@ static void init_selinux_config(void)
 
 	if (selinux_policyroot)
 		return;
-	if (access(SELINUXDIR, F_OK) != 0) {
-		selinux_policyroot = SECURITYDIR;
-		selinux_rootpath = SECURITYDIR;
-		use_compat_file_path = 1;
-		return;
-	}
 
 	selinux_rootpath = SELINUXDIR;
 	fp = fopen(SELINUXCONFIG, "r");
@@ -207,10 +173,6 @@ static void init_selinux_config(void)
 					    sizeof(REQUIRESEUSERS) - 1)) {
 				value = buf_p + sizeof(REQUIRESEUSERS) - 1;
 				intptr = &require_seusers;
-			} else if (!strncmp(buf_p, CACHETRANSTAG,
-					    sizeof(CACHETRANSTAG) - 1)) {
-				value = buf_p + sizeof(CACHETRANSTAG) - 1;
-				intptr = &cache_trans;
 			} else {
 				continue;
 			}
@@ -243,7 +205,6 @@ static void init_selinux_config(void)
 			     file_path_suffixes_idx[i])
 		    == -1)
 			return;
-	use_compat_file_path = 0;
 }
 
 static void fini_selinux_policyroot(void) __attribute__ ((destructor));
@@ -251,10 +212,6 @@ static void fini_selinux_policyroot(void) __attribute__ ((destructor));
 static void fini_selinux_policyroot(void)
 {
 	int i;
-	if (use_compat_file_path) {
-		selinux_policyroot = NULL;
-		return;
-	}
 	free(selinux_policyroot);
 	selinux_policyroot = NULL;
 	for (i = 0; i < NEL; i++) {
@@ -267,10 +224,7 @@ static void fini_selinux_policyroot(void)
 
 static const char *get_path(int idx)
 {
-	if (!use_compat_file_path)
-		return file_paths[idx];
-
-	return compat_file_path_data.str + compat_file_path_idx[idx];
+	return file_paths[idx];
 }
 
 const char *selinux_default_type_path()
@@ -299,6 +253,13 @@ const char *selinux_default_context_path()
 
 hidden_def(selinux_default_context_path)
 
+const char *selinux_securetty_types_path()
+{
+	return get_path(SECURETTY_TYPES);
+}
+
+hidden_def(selinux_securetty_types_path)
+
 const char *selinux_failsafe_context_path()
 {
 	return get_path(FAILSAFE_CONTEXT);
