@@ -24,8 +24,6 @@ static __thread security_context_t prev_t2r_raw = NULL;
 static __thread security_context_t prev_r2t_trans = NULL;
 static __thread security_context_t prev_r2t_raw = NULL;
 
-int cache_trans hidden = 1;
-
 /*
  * setransd_open
  *
@@ -203,39 +201,16 @@ static int trans_to_raw_context(char *trans, char **rawp)
 
 hidden void fini_context_translations(void)
 {
-	if (cache_trans) {
-		free(prev_r2t_trans);
-		free(prev_r2t_raw);
-		free(prev_t2r_trans);
-		free(prev_t2r_raw);
-	}
+	free(prev_r2t_trans);
+	free(prev_r2t_raw);
+	free(prev_t2r_trans);
+	free(prev_t2r_raw);
 }
 
 hidden int init_context_translations(void)
 {
-	int ret, fd;
-	int32_t ret_val;
-	char *out = NULL;
-
 	mls_enabled = is_selinux_mls_enabled();
-	if (!mls_enabled)
-		return 0;
-
-	fd = setransd_open();
-	if (fd < 0)
-		return fd;
-
-	ret = send_request(fd, SETRANS_INIT, NULL, NULL);
-	if (ret)
-		goto out;
-
-	ret = receive_response(fd, SETRANS_INIT, &out, &ret_val);
-	free(out);
-	if (!ret)
-		ret = ret_val;
-      out:
-	close(fd);
-	return ret;
+	return 0;
 }
 
 int selinux_trans_to_raw_context(security_context_t trans,
@@ -251,29 +226,26 @@ int selinux_trans_to_raw_context(security_context_t trans,
 		goto out;
 	}
 
-	if (cache_trans) {
-		if (prev_t2r_trans && strcmp(prev_t2r_trans, trans) == 0) {
-			*rawp = strdup(prev_t2r_raw);
-		} else {
-			free(prev_t2r_trans);
-			prev_t2r_trans = NULL;
-			free(prev_t2r_raw);
-			prev_t2r_raw = NULL;
-			if (trans_to_raw_context(trans, rawp))
-				*rawp = strdup(trans);
-			if (*rawp) {
-				prev_t2r_trans = strdup(trans);
-				if (!prev_t2r_trans)
-					goto out;
-				prev_t2r_raw = strdup(*rawp);
-				if (!prev_t2r_raw) {
-					free(prev_t2r_trans);
-					prev_t2r_trans = NULL;
-				}
+	if (prev_t2r_trans && strcmp(prev_t2r_trans, trans) == 0) {
+		*rawp = strdup(prev_t2r_raw);
+	} else {
+		free(prev_t2r_trans);
+		prev_t2r_trans = NULL;
+		free(prev_t2r_raw);
+		prev_t2r_raw = NULL;
+		if (trans_to_raw_context(trans, rawp))
+			*rawp = strdup(trans);
+		if (*rawp) {
+			prev_t2r_trans = strdup(trans);
+			if (!prev_t2r_trans)
+				goto out;
+			prev_t2r_raw = strdup(*rawp);
+			if (!prev_t2r_raw) {
+				free(prev_t2r_trans);
+				prev_t2r_trans = NULL;
 			}
 		}
-	} else if (trans_to_raw_context(trans, rawp))
-		*rawp = strdup(trans);
+	}
       out:
 	return *rawp ? 0 : -1;
 }
@@ -293,29 +265,26 @@ int selinux_raw_to_trans_context(security_context_t raw,
 		goto out;
 	}
 
-	if (cache_trans) {
-		if (prev_r2t_raw && strcmp(prev_r2t_raw, raw) == 0) {
-			*transp = strdup(prev_r2t_trans);
-		} else {
-			free(prev_r2t_raw);
-			prev_r2t_raw = NULL;
-			free(prev_r2t_trans);
-			prev_r2t_trans = NULL;
-			if (raw_to_trans_context(raw, transp))
-				*transp = strdup(raw);
-			if (*transp) {
-				prev_r2t_raw = strdup(raw);
-				if (!prev_r2t_raw)
-					goto out;
-				prev_r2t_trans = strdup(*transp);
-				if (!prev_r2t_trans) {
-					free(prev_r2t_raw);
-					prev_r2t_raw = NULL;
-				}
+	if (prev_r2t_raw && strcmp(prev_r2t_raw, raw) == 0) {
+		*transp = strdup(prev_r2t_trans);
+	} else {
+		free(prev_r2t_raw);
+		prev_r2t_raw = NULL;
+		free(prev_r2t_trans);
+		prev_r2t_trans = NULL;
+		if (raw_to_trans_context(raw, transp))
+			*transp = strdup(raw);
+		if (*transp) {
+			prev_r2t_raw = strdup(raw);
+			if (!prev_r2t_raw)
+				goto out;
+			prev_r2t_trans = strdup(*transp);
+			if (!prev_r2t_trans) {
+				free(prev_r2t_raw);
+				prev_r2t_raw = NULL;
 			}
 		}
-	} else if (raw_to_trans_context(raw, transp))
-		*transp = strdup(raw);
+	}
       out:
 	return *transp ? 0 : -1;
 }
